@@ -132,8 +132,7 @@ def peel_off_esc_code(s):
 
 def token_type(info):
     # type: (Token) -> Optional[List[Dict[Text, Union[Text, bool, None]]]]
-    """
-    """
+
     if info["command"] == "m":
         # The default action for ESC[m is to act like ESC[0m
         # Ref: https://en.wikipedia.org/wiki/ANSI_escape_code#CSI_codes
@@ -144,7 +143,7 @@ def token_type(info):
             if value in FG_NUMBER_TO_COLOR: tokens.append({'fg':FG_NUMBER_TO_COLOR[value]})
             if value in BG_NUMBER_TO_COLOR: tokens.append({'bg':BG_NUMBER_TO_COLOR[value]})
             if value in NUMBER_TO_STYLE: tokens.append({NUMBER_TO_STYLE[value]:True})
-            if value == RESET_ALL: tokens.append(dict(dict((k, None) for k in STYLES), **{'fg':None, 'bg':None}))
+            if value == RESET_ALL: tokens.append(dict({k: None for k in STYLES}, **{'fg':None, 'bg':None}))
             if value == RESET_FG: tokens.append({'fg':None})
             if value == RESET_BG: tokens.append({'bg':None})
         # fmt: on
@@ -164,4 +163,4 @@ if __name__ == "__main__":
     doctest.testmod()
     # print(peel_off_esc_code('[2Astuff'))
     # print(peel_off_esc_code('Amore'))
-    print((repr(parse("[31mstuff is the best[32myay"))))
+    print(repr(parse("[31mstuff is the best[32myay")))
